@@ -63,8 +63,8 @@ KWD::Window::Window (QWidget *parent,
 		     int     w,
 		     int     h): QWidget (parent, 0),
     mType (type),
-    mClientId (clientId),
     mFrame (0),
+    mClientId (clientId),
     mSelectedId (0),
     mDecor (0),
     mPixmap (0),
@@ -89,13 +89,15 @@ KWD::Window::Window (QWidget *parent,
 
     if (mType == Normal || mType == Switcher)
     {
-	KWin::WindowInfo wInfo = KWin::windowInfo (mClientId, NET::WMState, 0);
+	KWin::WindowInfo wInfo = KWin::windowInfo (mClientId, NET::WMState |
+			                           NET::WMVisibleName, 0);
 
 	mState = wInfo.state ();
 
 	if (mType == Normal)
 	{
-	    mName = KWin::readNameProperty (mClientId, XA_WM_NAME);
+	    mName = wInfo.visibleName ();
+
 	    mIcons = QIconSet (KWin::icon (mClientId, 16, 16, TRUE),
 			       KWin::icon (mClientId, 32, 32, TRUE));
 	    mOpacity = readPropertyShort (mClientId, Atoms::netWmWindowOpacity,
@@ -932,7 +934,7 @@ KWD::Window::updateShadow (void)
     /* use default shadow if such exist */
     if (!mUniqueHorzShape && !mUniqueVertShape)
     {
-	mShadow = Decorator::defaultWindowShadow (&mContext);
+	mShadow = Decorator::defaultWindowShadow (&mContext, &mBorder);
 	if (mShadow)
 	    decor_shadow_reference (mShadow);
     }
@@ -1377,36 +1379,32 @@ KWD::Window::updateProperty (void)
 
     if (mType == Normal || mType == Switcher)
     {
-	int topXOffset = w / 2;
+	int	topXOffset = w / 2;
+	QWidget *widget = mDecor->widget ();
+	int	x;
 
-	if (mDecor)
+	x = w - mContext.left_space - mContext.left_corner_space;
+	if (x > topXOffset)
+	    topXOffset = x;
+
+	if (widget)
 	{
-	    QWidget *widget = mDecor->widget ();
-	    int	    x;
+	    const QObjectList *children = widget->children ();
 
-	    x = w - mContext.left_space - mContext.left_corner_space;
-	    if (x > topXOffset)
-		topXOffset = x;
-
-	    if (widget)
+	    if (children)
 	    {
-		const QObjectList *children = widget->children ();
+		QWidget *child;
 
-		if (children)
+		for (QObjectListIt it(*children); it.current (); ++it)
 		{
-		    QWidget *child;
+		    if (!it.current ()->isWidgetType ())
+			continue;
 
-		    for (QObjectListIt it(*children); it.current (); ++it)
-		    {
-			if (!it.current ()->isWidgetType ())
-			    continue;
+		    child = static_cast <QWidget *> (it.current ());
 
-			child = static_cast <QWidget *> (it.current ());
-
-			x = child->x () - mBorder.left - 2;
-			if (x > w / 2 && x < topXOffset)
-			    topXOffset = x;
-		    }
+		    x = child->x () - mBorder.left - 2;
+		    if (x > w / 2 && x < topXOffset)
+			topXOffset = x;
 		}
 	    }
 	}
@@ -1476,24 +1474,24 @@ KWD::Window::updateSelected (WId selectedId)
 void
 KWD::Window::updateWindowGeometry (void)
 {
-    unsigned int w, h, bw, d;
-    int          x, y;
-    XID		 root;
+    KWin::WindowInfo wInfo = KWin::windowInfo (mClientId, NET::WMGeometry);
+    QRect	     geometry = wInfo.geometry ();
 
-    KWD::trapXError ();
-    XGetGeometry (qt_xdisplay (), mClientId, &root, &x, &y, &w, &h, &bw, &d);
-    if (KWD::popXError ())
-	return;
+    if (mGeometry.width ()  != geometry.width () ||
+	mGeometry.height () != geometry.height ())
+    {
+	mGeometry = geometry;
 
-    w += bw * 2;
-    h += bw * 2;
+	if (resizeDecoration ())
+	    return;
+    }
+    else
+    {
+	mGeometry = geometry;
+    }
 
-    mGeometry = QRect (x, y, w, h);
-    if (resizeDecoration ())
-	return;
-
-    move (x + ROOT_OFF_X - mBorder.left,
-	  y + ROOT_OFF_Y - mBorder.top);
+    move (mGeometry.x () + ROOT_OFF_X - mBorder.left,
+	  mGeometry.y () + ROOT_OFF_Y - mBorder.top);
 }
 
 void
@@ -1693,17 +1691,21 @@ KWD::Window::updateState (void)
 void
 KWD::Window::updateName (void)
 {
+    KWin::WindowInfo wInfo;
+    WId              window;
+
     if (mType == Switcher)
     {
 	if (!mSelectedId)
 	    return;
-
-	mName = KWin::readNameProperty (mSelectedId, XA_WM_NAME);
+	window = mSelectedId;
     }
     else
-    {
-	mName = KWin::readNameProperty (mClientId, XA_WM_NAME);
-    }
+	window = mClientId;
+
+    wInfo = KWin::windowInfo (window, NET::WMVisibleName, 0);
+
+    mName = wInfo.visibleName ();
 
     mDecor->captionChange ();
 }
