@@ -30,6 +30,8 @@
 
 #include <compiz.h>
 
+static CompMetadata inotifyMetadata;
+
 static int displayPrivateIndex;
 
 typedef struct _CompInotifyWatch {
@@ -80,7 +82,7 @@ inotifyProcessEvents (void *data)
 	    event = (struct inotify_event *) &buf[i];
 
 	    for (iw = id->watch; iw; iw = iw->next)
-		if (iw->handle == event->wd)
+		if (iw->wd == event->wd)
 		    break;
 
 	    if (iw)
@@ -90,7 +92,12 @@ inotifyProcessEvents (void *data)
 			break;
 
 		if (fw)
-		    (*fw->callBack) (event->name, fw->closure);
+		{
+		    if (event->len)
+			(*fw->callBack) (event->name, fw->closure);
+		    else
+			(*fw->callBack) (NULL, fw->closure);
+		}
 	    }
 
 	    i += sizeof (*event) + event->len;
@@ -233,9 +240,18 @@ inotifyFiniDisplay (CompPlugin  *p,
 static Bool
 inotifyInit (CompPlugin *p)
 {
+    if (!compInitPluginMetadataFromInfo (&inotifyMetadata, p->vTable->name,
+					 0, 0, 0, 0))
+	return FALSE;
+
     displayPrivateIndex = allocateDisplayPrivateIndex ();
     if (displayPrivateIndex < 0)
+    {
+	compFiniMetadata (&inotifyMetadata);
 	return FALSE;
+    }
+
+    compAddMetadataFromFile (&inotifyMetadata, p->vTable->name);
 
     return TRUE;
 }
@@ -243,8 +259,8 @@ inotifyInit (CompPlugin *p)
 static void
 inotifyFini (CompPlugin *p)
 {
-    if (displayPrivateIndex >= 0)
-	freeDisplayPrivateIndex (displayPrivateIndex);
+    freeDisplayPrivateIndex (displayPrivateIndex);
+    compFiniMetadata (&inotifyMetadata);
 }
 
 static int
@@ -254,11 +270,16 @@ inotifyGetVersion (CompPlugin *plugin,
     return ABIVERSION;
 }
 
+static CompMetadata *
+inotifyGetMetadata (CompPlugin *plugin)
+{
+    return &inotifyMetadata;
+}
+
 CompPluginVTable inotifyVTable = {
     "inotify",
-    "Inotify",
-    "File change notification plugin",
     inotifyGetVersion,
+    inotifyGetMetadata,
     inotifyInit,
     inotifyFini,
     inotifyInitDisplay,
@@ -270,11 +291,7 @@ CompPluginVTable inotifyVTable = {
     0, /* GetDisplayOptions */
     0, /* SetDisplayOption */
     0, /* GetScreenOptions */
-    0, /* SetScreenOption */
-    0, /* Deps */
-    0, /* nDeps */
-    0, /* Features */
-    0  /* nFeatures */
+    0  /* SetScreenOption */
 };
 
 CompPluginVTable *
