@@ -29,6 +29,8 @@
 
 #include <compiz.h>
 
+#define DEG2RAD (M_PI / 180.0f)
+
 ScreenPaintAttrib defaultScreenPaintAttrib = {
     0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, -DEFAULT_Z_CAMERA
 };
@@ -47,7 +49,7 @@ donePaintScreen (CompScreen *screen) {}
 void
 applyScreenTransform (CompScreen	      *screen,
 		      const ScreenPaintAttrib *sAttrib,
-		      int		      output,
+		      CompOutput	      *output,
 		      CompTransform	      *transform)
 {
     matrixTranslate (transform,
@@ -58,41 +60,41 @@ applyScreenTransform (CompScreen	      *screen,
 		  sAttrib->xRotate, 0.0f, 1.0f, 0.0f);
     matrixRotate (transform,
 		  sAttrib->vRotate,
-		  1.0f - sAttrib->xRotate / 90.0f,
+		  cosf (sAttrib->xRotate * DEG2RAD),
 		  0.0f,
-		  sAttrib->xRotate / 90.0f);
+		  sinf (sAttrib->xRotate * DEG2RAD));
     matrixRotate (transform,
 		  sAttrib->yRotate, 0.0f, 1.0f, 0.0f);
 }
 
 void
 transformToScreenSpace (CompScreen    *screen,
-			int	      output,
+			CompOutput    *output,
 			float         z,
 			CompTransform *transform)
 {
     matrixTranslate (transform, -0.5f, -0.5f, z);
     matrixScale (transform,
-		 1.0f  / screen->outputDev[output].width,
-		 -1.0f / screen->outputDev[output].height,
+		 1.0f  / output->width,
+		 -1.0f / output->height,
 		 1.0f);
     matrixTranslate (transform,
-		     -screen->outputDev[output].region.extents.x1,
-		     -screen->outputDev[output].region.extents.y2,
+		     -output->region.extents.x1,
+		     -output->region.extents.y2,
 		     0.0f);
 }
 
 void
 prepareXCoords (CompScreen *screen,
-		int	   output,
+		CompOutput *output,
 		float      z)
 {
     glTranslatef (-0.5f, -0.5f, z);
-    glScalef (1.0f  / screen->outputDev[output].width,
-	      -1.0f / screen->outputDev[output].height,
+    glScalef (1.0f  / output->width,
+	      -1.0f / output->height,
 	      1.0f);
-    glTranslatef (-screen->outputDev[output].region.extents.x1,
-		  -screen->outputDev[output].region.extents.y2,
+    glTranslatef (-output->region.extents.x1,
+		  -output->region.extents.y2,
 		  0.0f);
 }
 
@@ -148,16 +150,18 @@ paintCursor (CompCursor		 *c,
    transformed screen case should be made optional for those who do
    see a difference. */
 static void
-paintScreenRegion (CompScreen	       *screen,
+paintOutputRegion (CompScreen	       *screen,
 		   const CompTransform *transform,
 		   Region	       region,
-		   int		       output,
+		   CompOutput	       *output,
 		   unsigned int	       mask)
 {
     static Region tmpRegion = NULL;
     CompWindow    *w;
     CompCursor	  *c;
     int		  count, windowMask, backgroundMask;
+    CompWindow	  *fullscreenWindow = NULL;
+    CompWalker    walk;
 
     if (!tmpRegion)
     {
@@ -181,8 +185,10 @@ paintScreenRegion (CompScreen	       *screen,
 
     XSubtractRegion (region, &emptyRegion, tmpRegion);
 
+    (*screen->initWindowWalker) (screen, &walk);
+
     /* detect occlusions */
-    for (w = screen->reverseWindows; w; w = w->prev)
+    for (w = (*walk.last) (screen); w; w = (*walk.prev) (w))
     {
 	if (w->destroyed)
 	    continue;
@@ -199,6 +205,8 @@ paintScreenRegion (CompScreen	       *screen,
 	if ((*screen->paintWindow) (w, &w->paint, transform, tmpRegion,
 				    PAINT_WINDOW_OCCLUSION_DETECTION_MASK))
 	{
+	    XSubtractRegion (tmpRegion, w->region, tmpRegion);
+
 	    /* unredirect top most fullscreen windows. */
 	    if (count == 0					      &&
 		!REGION_NOT_EMPTY (tmpRegion)			      &&
@@ -206,9 +214,8 @@ paintScreenRegion (CompScreen	       *screen,
 		XEqualRegion (w->region, &screen->region))
 	    {
 		unredirectWindow (w);
+		fullscreenWindow = w;
 	    }
-
-	    XSubtractRegion (tmpRegion, w->region, tmpRegion);
 	}
 
 	count++;
@@ -217,11 +224,14 @@ paintScreenRegion (CompScreen	       *screen,
     (*screen->paintBackground) (screen, tmpRegion, backgroundMask);
 
     /* paint all windows from bottom to top */
-    for (w = screen->windows; w; w = w->next)
+    for (w = (*walk.first) (screen); w; w = (*walk.next) (w))
     {
 	if (w->destroyed)
 	    continue;
 
+	if (w == fullscreenWindow)
+	    continue;
+
 	if (!w->shaded)
 	{
 	    if (w->attrib.map_state != IsViewable || !w->damaged)
@@ -231,6 +241,9 @@ paintScreenRegion (CompScreen	       *screen,
 	(*screen->paintWindow) (w, &w->paint, transform, w->clip, windowMask);
     }
 
+    if (walk.fini)
+	(*walk.fini) (screen, &walk);
+
     /* paint cursors */
     for (c = screen->cursors; c; c = c->next)
 	(*screen->paintCursor) (c, transform, tmpRegion, 0);
@@ -241,11 +254,11 @@ paintScreenRegion (CompScreen	       *screen,
 			 PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS_MASK)
 
 void
-paintTransformedScreen (CompScreen		*screen,
+paintTransformedOutput (CompScreen		*screen,
 			const ScreenPaintAttrib *sAttrib,
 			const CompTransform	*transform,
 			Region			region,
-			int			output,
+			CompOutput		*output,
 			unsigned int		mask)
 {
     CompTransform sTransform = *transform;
@@ -264,11 +277,11 @@ paintTransformedScreen (CompScreen		*screen,
 	GLdouble p1[2] = { region->extents.x1, h - region->extents.y2 };
 	GLdouble p2[2] = { region->extents.x2, h - region->extents.y1 };
 
-	GLdouble halfW = screen->outputDev[output].width / 2.0;
-	GLdouble halfH = screen->outputDev[output].height / 2.0;
+	GLdouble halfW = output->width / 2.0;
+	GLdouble halfH = output->height / 2.0;
 
-	GLdouble cx = screen->outputDev[output].region.extents.x1 + halfW;
-	GLdouble cy = (h - screen->outputDev[output].region.extents.y2) + halfH;
+	GLdouble cx = output->region.extents.x1 + halfW;
+	GLdouble cy = (h - output->region.extents.y2) + halfH;
 
 	GLdouble top[4]    = { 0.0, halfH / (cy - p1[1]), 0.0, 0.5 };
 	GLdouble bottom[4] = { 0.0, halfH / (cy - p2[1]), 0.0, 0.5 };
@@ -293,7 +306,7 @@ paintTransformedScreen (CompScreen		*screen,
 
 	glLoadMatrixf (sTransform.m);
 
-	paintScreenRegion (screen, &sTransform, region, output, mask);
+	paintOutputRegion (screen, &sTransform, region, output, mask);
 
 	glDisable (GL_CLIP_PLANE0);
 	glDisable (GL_CLIP_PLANE1);
@@ -310,18 +323,18 @@ paintTransformedScreen (CompScreen		*screen,
 	glPushMatrix ();
 	glLoadMatrixf (sTransform.m);
 
-	paintScreenRegion (screen, &sTransform, region, output, mask);
+	paintOutputRegion (screen, &sTransform, region, output, mask);
 
 	glPopMatrix ();
     }
 }
 
 Bool
-paintScreen (CompScreen		     *screen,
+paintOutput (CompScreen		     *screen,
 	     const ScreenPaintAttrib *sAttrib,
 	     const CompTransform     *transform,
 	     Region		     region,
-	     int		     output,
+	     CompOutput		     *output,
 	     unsigned int	     mask)
 {
     CompTransform sTransform = *transform;
@@ -332,9 +345,9 @@ paintScreen (CompScreen		     *screen,
 	{
 	    if (mask & PAINT_SCREEN_FULL_MASK)
 	    {
-		region = &screen->outputDev[output].region;
+		region = &output->region;
 
-		(*screen->paintTransformedScreen) (screen, sAttrib,
+		(*screen->paintTransformedOutput) (screen, sAttrib,
 						   &sTransform, region,
 						   output, mask);
 
@@ -348,8 +361,8 @@ paintScreen (CompScreen		     *screen,
     }
     else if (mask & PAINT_SCREEN_FULL_MASK)
     {
-	(*screen->paintTransformedScreen) (screen, sAttrib, &sTransform,
-					   &screen->outputDev[output].region,
+	(*screen->paintTransformedOutput) (screen, sAttrib, &sTransform,
+					   &output->region,
 					   output, mask);
 
 	return TRUE;
@@ -364,7 +377,7 @@ paintScreen (CompScreen		     *screen,
     glPushMatrix ();
     glLoadMatrixf (sTransform.m);
 
-    paintScreenRegion (screen, &sTransform, region, output, mask);
+    paintOutputRegion (screen, &sTransform, region, output, mask);
 
     glPopMatrix ();
 
